searchData={"items":[{"type":"module","title":"Truly","doc":"Create a truth table to consolidate complex boolean conditional logic.\n\n`Truly` provides a convenient and human-readable way to store complex conditional logic trees.\nYou can immediately use `Truly.evaluate/2` to evaluate the truth table or pass the truth table\naround for repeat use.\n\nYou might find this useful for things like feature flags, where depending on the combination\nof boolean flags you want different behaviors or paths.\n\nThis can also make the design much more self-documented, where the intent behind a large\nlogic ladder becomes quite clear.","ref":"Truly.html"},{"type":"module","title":"Installation - Truly","doc":"The package can be installed\nby adding `truly` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:truly, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"Truly.html#module-installation"},{"type":"module","title":"Usage - Truly","doc":"First, you must `import Truly`. Then you have the\n`~TRULY` sigil available.\n\nAll column names and result values must be valid (and existing) atoms.\n\nTable cells can only be boolean values.\n\nYou must provide an exhaustive truth table, meaning that you provide\neach combination of column values.","ref":"Truly.html#module-usage"},{"type":"module","title":"Basic Example - Truly","doc":"```elixir\nimport Truly\ncolumns = [:flag_a, :flag_b, :flag_c]\ncategories = [:cat1, :cat2, :cat3]\n\n{:ok, tt} = ~TRULY\"\"\"\n| flag_a |  flag_b | flag_c   |          |  \n|--------|---------|----------|----------|\n|   0    |    0    |     0    |   cat1   |\n|   0    |    0    |     1    |   cat1   |\n|   0    |    1    |     0    |   cat2   |\n|   0    |    1    |     1    |   cat1   |\n|   1    |    0    |     0    |   cat3   |\n|   1    |    0    |     1    |   cat1   |\n|   1    |    1    |     0    |   cat2   |\n|   1    |    1    |     1    |   cat3   |\n\"\"\"\n\nTruly.evaluate!(tt,[flag_a: 1, flag_b: 1, flag_c: 1])\n\nflag_a = 0\nflag_b = 1\nflag_c = 0\n\nTruly.evaluate(tt,binding())\n```","ref":"Truly.html#module-basic-example"},{"type":"module","title":"Practical Example - Truly","doc":"Imagine you're writing the backend for your social media app\ncalled `PitterPatter`. You want to allow users to direct message\neach other, but you want to enforce certain rules around this.\n\nYou have the following struct representing your `User`:\n\n```elixir\ndefmodule User do\n  defstruct [:dms_open, :locked]\nend\n```\n\nYou want to control when messages are allowed to be sent according to\nthe sender's `:locked` account status, the receiver's `:dms_open` setting,\nas well as if the two are friends.\n\nDifferent combinations of these result in different behavior.\n\nWe can define the truth table, and since the result column\ncan be any atom, we can directly pass the function that we want\nto call:\n\n```elixir\ndefmodule PitterPatter do\n  import Truly\n\n  def are_friends(_user1, _user2), do: Enum.random([true,false]) |> IO.inspect(label: \"Are friends?\")\n\n  # We must specify these atoms before the truth table since the atoms must exist already\n  @flags  [:dms_open, :locked]\n\n  # Have different functions for different behaviors. You could imagine there\n  # can be any number of these <= # rows\n  def send_message(_sender,_receiver,_message), do: \"Message Sent!\"\n  def deny_message(_sender,_receiver,_message), do: \"Sorry, you can't send that message!\"\n\n\n  # Specify our truth table\n  # For the sake of simplicity we stick to 3 variables\n  # Also notice that you can use `true, false, 1, 0` in the cells\n  @tt ~TRULY\"\"\"\n  | dms_open |  are_friends |  locked  |                   |   \n  |----------|--------------|----------|-------------------|\n  |     0    |       0      |     0    |    deny_message   |\n  |     0    |     false    |     1    |    deny_message   |\n  |     0    |       1      |     0    |    send_message   |\n  |     0    |       1      |     1    |    deny_message   |\n  |     1    |       0      |     0    |    send_message   |\n  |     1    |       0      |     1    |    deny_message   |\n  |     1    |       true   |     0    |    send_message   |\n  |     1    |       1      |     1    |    deny_message   |\n  \"\"\"r # <- Notice the `r` modifier after the table\n       # This is effectively like a `!` function, that will\n       # unpack the return tuple and raise on error\n  def direct_message(sender, receiver, message) do\n    table = @tt \n    flags = \n      [\n        dms_open: receiver.dms_open, \n        are_friends: are_friends(sender,receiver),\n        locked: sender.locked\n      ]\n    apply(__MODULE__,Truly.evaluate!(table,flags),[sender,receiver,message])\n  end\nend\n```\n\nAnd just like that, a call to `Truly.evaluate!` performs all of the various checks\nneeded as well as routes to the appropriate function depending on the state passed in.\n\nLet's see how we would use this Let's set up some `User`s:\n\n```elixir\nsender = %User{dms_open: true, locked: false}\nreceiver = %User{dms_open: true, locked: false}\n```\n\nAnd now you can run `PitterPatter.direct_message`, and you will see that\nas the `:are_friends` status changes (since it's determined randomly above),\nthe result changes according to the rows in the truth table.\n\n```elixir\nPitterPatter.direct_message(sender, receiver, \"Hey, can you talk?\")\n```","ref":"Truly.html#module-practical-example"},{"type":"function","title":"Truly.evaluate/2","doc":"Evaluated the truth table with the given bindings.\n\nThe bindings are any keyword list that have the corresponding column names to the\ntruth table, so you can manually pass them or pass `binding()` if the variables are\ndefined therein.","ref":"Truly.html#evaluate/2"},{"type":"function","title":"Truly.evaluate!/2","doc":"Same as `evaluate/2` but raises on error","ref":"Truly.html#evaluate!/2"},{"type":"function","title":"Truly.sigil_TRULY/2","doc":"Creates a new `Truly` struct which stores the truth table. This can later be\nevaluated against a list of bindings to get the truth value.","ref":"Truly.html#sigil_TRULY/2"},{"type":"function","title":"Modifiers - Truly.sigil_TRULY/2","doc":"* `r` - This is effectively like a `!` function, that will unpack the return tuple and raise on error","ref":"Truly.html#sigil_TRULY/2-modifiers"},{"type":"type","title":"Truly.t/0","doc":"","ref":"Truly.html#t:t/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}